//
// This program demonstrates how to use the TBarcode Library.
//
// IMPORTANT: A valid license is required to test the callback functions.
// Callback functions are disabled without a valid license.
// Temporary licenses for testing are available from mailto:barcode@tec-it.com.
//

#ifdef WIN32
  #include "tbarcode.h"
#elif TEC_MAC_64
  #include <TBarCode64/tbarcode.h>
#elif TEC_MAC
  #include <TBarCode/tbarcode.h>
#else // LINUX or UNIX
  #include <libtbarcode11/tbarcode.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef TEC_MAC
  #define PATH_TBARCODE "/tecit/TBarCode11"
#else
  #define PATH_TBARCODE "/usr/local/share/tbarcode11"
#endif


// prototype of custom callback function
ERRCODE CALLBACK CustomDrawRow(VOID* customData, t_BarCode* barcode, HDC drawDC, HDC targetDC, DOUBLE x, DOUBLE y, DOUBLE width, DOUBLE height);
ERRCODE CALLBACK CustomDrawBar(VOID* customData, HDC drawDC, DOUBLE x, DOUBLE y, DOUBLE width, DOUBLE height);
ERRCODE CALLBACK CustomDrawText(VOID* customData, HDC drawDC, LPLOGFONTA font, DOUBLE x, DOUBLE y, e_BCAlign alignment, LPCSTR text, INT numberOfCharacters, LPRECTD drawArea, LONG textIndex);


int main(int argc, char* argv[])
{
  ERRCODE eCode;                      // return value of TBarCode functions
  RECT rect;                          // bounding rectangle of the barcode
  t_BarCode* pBarCode = NULL;         // barcode handle
  const char* demo = "Demo12345";     // data, which will be encoded in barcode
  
  //
  // ------ Initialize TBarCode Library
  //

#if !defined(WIN32)
  // Initialize library (required in LINUX/UNIX)
  eCode = BCInitLibrary(PATH_TBARCODE);
  printf ("BCInitLibrary - eCode = %ld\n", eCode);
#endif

  // Set License Key
  // 
  // IMPORTANT:
  // A valid license key is required to test the callback functions!!
  // Please request a temporary license key (for testing) from mailto:sales@tec-it.com
  // After you have received the email with the license key, insert the key to the function parameters below:
  // 
  //
  //  eCode = BCLicenseMe("insert licensee name here", 
  //                       eLicKindSite,
  //                       1, 
  //                       "insert license key here",
  //                       eLicProd2D);

  //
  // ------ Set BarCode parameters
  //

  // Allocate bar code structure
  eCode = BCAlloc(&pBarCode);
  printf ("BCAlloc - eCode = %d\n", eCode);
  
  // Set size of font
  //eCode = BCSetFontHeight(pBarCode, 10);
  //printf ("BCSetFontHeight - eCode = %d\n", eCode);

  // Set bar code type to 'Data Matrix'
  eCode = BCSetBCType(pBarCode, eBC_DataMatrix);
  printf ("BCSetBCType - eCode = %d\n", eCode);

  // Set content of bar code
  eCode = BCSetText(pBarCode, demo, (LONG)strlen(demo));
  printf ("BCSetText - eCode = %d\n", eCode);

  // Create barcode
  eCode = BCCreate(pBarCode);
  printf("BCCreate - eCode = %d\n", eCode);

  //
  // ------ Create BarCode using callback functions
  //

  // Set size of barcode [1/1000mm]
  // bottom, left corner:
  rect.bottom = 0;        // 0 mm
  rect.left   = 0;        // 0 mm
  
  // top, right corner:
  rect.top    = 30000;    // 30 mm
  rect.right  = 50000;    // 50 mm
  
  // Set callback
  eCode = BCSetFuncDrawRow(pBarCode, CustomDrawRow);
  printf("Enabling BCSetFuncDrawRow: %d\n", eCode);

  eCode = BCSetFuncDrawBar(pBarCode, CustomDrawBar);
  printf("Enabling BCSetFuncDrawBar: %d\n", eCode);

  eCode = BCSetFuncDrawText(pBarCode, CustomDrawText);
  printf("Enabling BCSetFuncDrawText: %d\n", eCode);

  printf("Pattern Description:\n");
  printf("====================\n");
  printf("An upper case 'A' is a black pixel\n");
  printf("An lower case 'a' is a white pixel\n");
  printf("You can find the pattern of a DataMatrix code below:\n\n");

  eCode = BCDrawCB(pBarCode, NULL, &rect);
  printf("BCDrawCB: %d\n", eCode );

  //
  // ------ Clean up
  //

  // Free barcode structure
  eCode = BCFree(pBarCode);
  printf("BCFree - eCode = %d\n", eCode);

#if !defined(WIN32)
  // Clean up (required in LINUX/UNIX)
  eCode = BCDeInitLibrary();
  printf("BCDeInitLibrary - eCode = %ld\n", eCode);
#endif

  return EXIT_SUCCESS;
}


// Custom callback function for that is called for each row
ERRCODE CALLBACK CustomDrawRow(VOID* customData, t_BarCode* barcode, HDC drawDC, HDC targetDC, DOUBLE x, DOUBLE y, DOUBLE width, DOUBLE height)
{
  LPCSTR strData;
  strData = BCGetMetaData(barcode);
  printf("RowPattern=%s\n", strData);
  return ErrOk;
}

// Custom callback function that is called for each bar
ERRCODE CALLBACK CustomDrawBar(VOID* customData, HDC drawDC, DOUBLE x, DOUBLE y, DOUBLE width, DOUBLE height)
{
  printf("DrawBar Callback: x=%f, y=%f, width=%f, height=%f \n", x, y, width, height);
  return ErrOk;
}

// Custom callback function that is called for each text
ERRCODE CALLBACK CustomDrawText(VOID* customData, HDC drawDC, LPLOGFONTA font, DOUBLE x, DOUBLE y, e_BCAlign alignment, LPCSTR text, INT numberOfCharacters, LPRECTD drawArea, LONG textIndex)
{
  int s = (font) ? font->lfHeight : -1;
  printf("DrawText Callback: x=%f, y=%f, size=%d, \"%s\", characters=%d\n", x, y, s, text, numberOfCharacters);
  return ErrOk;
}
